<?php
/**
 * Registro10SEFIP
 *
 * @package    Logic
 * @subpackage Registro/Sefip
 *
 * @author     alessandrobartels - 02/06/2011
 *
 */
class Registro10SEFIP extends RegistroSEFIP{

    /**
     *
     */
    const REGISTRO_10 = '10';

    // Verificar se estas constantes podem ir para nivel superior.
    /**
     *
     */
    const SIMPLES_NAO_OPTANTE = '1';
    const SIMPLES_OPTANTE = '2';
    const CARACTER_HIFEN = ' - ';

    private $_estrutura;
    private $_endereco;
    private $_nrInscricao;
    private $_tpInscricao;
    private $_nrRecolhimento;
    private $_fpas;
    private $_simples;
    private $_codCentralizadorFGTS;
    private $_CNPJCentralizador;
    private $_idTipoEmpr;
    private $_vrAliqRAT;

    /**
     * Preparacao para a criacao do Registro 10.
     *
     * @param EstruturamEntity $estrutura   Instancia de EstruturamEntity.
     * @param EnderecoEntity   $endereco    Instancia de EnderecoEntity.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     */
    public function geraRegistro10(EstruturamEntity $estrutura, EnderecoEntity $endereco){
        $this->_estrutura = $estrutura;
        $this->_endereco = $endereco;

        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_fpas = $this->_estrutura->historico->getCdfpas();
        $this->_idTipoEmpr = $this->_estrutura->historico->getIdtipoempr();

        $cnaefiscal = new CnaefiscalEntity();
        $this->_vrAliqRAT = $cnaefiscal->retornaAliquotaRAT($this->_estrutura->historico->getCdcnae());

        if($this->_estrutura->historico->getIdoptsimples() === 'N'){
            $this->_simples =  '1';
        }else{
            $this->_simples =  '2';
        }

        $this->_geraRegistro();
    }


    /**
     * Cria o Registro 10.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     */
    private  function _geraRegistro(){
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_10, $this->_tpInscricao, $this->_nrInscricao);

        // Bloco 4 | 018-053 | Preencher com zeros.
        $this->atribuiDado(parent::CARACTER_VAZIO, 36, parent::CARACTER_ZERO);

        // Bloco 5 | 054-093 | Razao Social.

        $this->_atribuiRazaoSocial($this->_estrutura->historico->getNmrazsocestrut());
//        $this->atribuiDado(
//            $this->_estrutura->historico->getNmrazsocestrut(), 40, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA
//        );

        // Bloco 6 | 094-143 | Endereco.
        $endereco = $this->_endereco->getDsendereco();
        $this->atribuiDado($endereco, 50, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 7 | 144-163 | Bairro.
        $bairro = $this->_endereco->bairro;
        $this->atribuiDado($bairro, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 8 | 164-171 | CEP. @todo Diferentes de 20000000, 30000000, 70000000 ou 80000000.
        $cep = $this->_endereco->getNrcependereco();
        $this->atribuiDado($cep, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 9 | 172-191 | Cidade.
        $cidade = $this->_endereco->municipio;
        $this->atribuiDado($cidade, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 10 | 192-193 | Estado.
        $estado = $this->_endereco->getSgestado();
        $this->atribuiDado($estado, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 11 | 194-205 | Telefone.
        $telefone = $this->_endereco->ddd.$this->_endereco->telefone;
        $this->atribuiDado($telefone, 12, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 12 | 206 | Indicador de alteracao de endere�o. S ou N.
        $indicador = parent::NAO;
        $this->atribuiDado($indicador, 1); /* @todo Verificar se pode vir como parametro no formulario. */

            // Bloco 13 | 207-213 | CNAE.
        $this->atribuiDado(
            $this->_estrutura->historico->getCdcnae(), 7, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA
        );

        // Bloco 14 | 214 | Id de alteracao do CNAE. @todo verificar layout.
        $this->atribuiDado(parent::INDICADORCNAE, 1, parent::INDICADORCNAE, parent::PREENCHE_DIREITA);
        /* @todo Verificar se pode vir como parametro no formulario. */

        // Bloco 15 | 215-216 | Aliquota RAT. @todo verificar layout.
        // @todo Aliquota deve vir do parametro de INSS.
        //@todo Passado fixo. Deve ser implementado no parametro de folha ou estrutura
        // @example Para a ORG 9 a aliquota � de 3%.
        $rat = 10;
        $this->_atribuiAliquotaRAT($rat); /*  @todo Verificar se pode vir como parametro no formulario. */

        // Bloco 16 | 217 | Codigo de Centralizacao.
        $this->_atribuiCodigoCentralizador();

        // Bloco 17 | 218 | SIMPLES.
        $this->_atribuiSimples($this->_estrutura->historico->getIdoptsimples());

        // Bloco 18 | 219-221 | FPAS. 
        $this->_atribuiFpas($this->_estrutura->historico->getCdfpas());

        // Bloco 19 | 222-225 | Codigo de outra entidades.
        $codigoTerceiros = $this->_estrutura->historico->getCdterceiro();
        $this->_atribuiCodigoTerceiros($codigoTerceiros);

        // Bloco 20 | 226-229 | Codigo pagamento GPS. @todo verificar layout.
        // @todo Codigo GPS deve vir do parametro de INSS.
        $codigoGPS = 2100;
        $this->_atribuiCodigoGPS($codigoGPS);

        // Bloco 21 | 230-234 | Percentual Filantropia. @todo verificar layout.
        //@todo Passado fixo. Deve ser implementado no parametro de folha ou estrutura
        $this->_atribuiPercentualFilantropia($percentualFilantropia);

        // Bloco 22 | 235-249 | Salario-Familia.
        /* @todo Pegar os valores do itemCalculo. Validar para competencia 13 */
        $this->_atribuiSalarioFamilia($vrSalarioFamilia);

        // Bloco 23 | 250-264 | Salario-Maternidade.
        /* @todo Pegar os valores do itemCalculo. Validar para competencia 13 */
        $this->_atribuiSalarioMaternidade($vrSalarioMaternidade);

        // Bloco 24 | 265-279 | Contribuicao Desc. Empregados Comp. 13. Preencher com '0'. Nao eh informado.
        $this->atribuiDado(parent::CARACTER_VAZIO, 15, parent::CARACTER_ZERO);

        // Bloco 25 | 280 | Indicador de valor +/-. Preencher com '0'. Nao eh informado.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ZERO);

        // Bloco 26 | 281-294 | Valor devido a previdencia Social Comp. 13. Preencher com '0'. Nao eh informado.
        $this->atribuiDado(parent::CARACTER_VAZIO, 14, parent::CARACTER_ZERO);

        // Bloco 27 | 295-297 | Banco. Preencher com ' '. Implementacao futura.
        $this->atribuiDado(parent::CARACTER_VAZIO, 3, parent::CARACTER_BRANCO);

        // Bloco 28 | 298-301 | Agencia. Preencher com ' '. Implementacao futura.
        $this->atribuiDado(parent::CARACTER_VAZIO, 4, parent::CARACTER_BRANCO);

        // Bloco 29 | 302-310 | Conta Corrente. Preencher com ' '. Implementacao futura.
        $this->atribuiDado(parent::CARACTER_VAZIO, 9, parent::CARACTER_BRANCO);

        // Bloco 30 | 311-355 | Zeros. Preencher com '0'. Implementacao futura.
        $this->atribuiDado(parent::CARACTER_VAZIO, 45, parent::CARACTER_ZERO);

        // Bloco 31 | 356-359 | Brancos. Preencher com ' '.
        $this->atribuiDado(parent::CARACTER_VAZIO, 4, parent::CARACTER_BRANCO);

        // Bloco 32 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);

    }


    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }


    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }


    public function setCNPJCentralizador($CNPJCentralizador)
    {
        $this->_CNPJCentralizador = $CNPJCentralizador;
    }


    public function setCodCentralizadorFGTS($codCentralizadorFGTS)
    {
        $this->_codCentralizadorFGTS = $codCentralizadorFGTS;
    }


    private function _atribuiRazaoSocial($razaosocial){
        $razaosocialtradado = str_replace(self::CARACTER_HIFEN, parent::CARACTER_BRANCO, $razaosocial);
        $this->atribuiDado($razaosocialtradado, 40, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /**
     * Atribui aliquota RAT.
     *
     * @param string $rat Aliquota RAT.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     *         alessandrobartels - 30/01/2012 - Implementacao de validacao do RAT.
     *         felipesantos      - 07/05/2015 - Busca alíquota do CNAE Fiscal
     */
    private function _atribuiAliquotaRAT($rat){

        if (!is_null($this->_vrAliqRAT)) {
            $rat = $this->_vrAliqRAT;
        }

        $codRecol = array(
            '145', '307', '317', '327', '337', '345', '640', '660',
        );

        // Nao pode ser informado p/ FPAS 604 com recolhimento de cod. 150 em compet. posteriores a 10/2001.
        // Nao pode ser informado para os codigos de recolhimento 145, 307, 317, 327, 337, 345, 640 e 660.
        if ((($this->_fpas == '604' && $this->_nrRecolhimento == '150') ||
             (array_search($this->_nrRecolhimento, $codRecol) == true))
        ) {
            $rat = parent::CARACTER_BRANCO.parent::CARACTER_BRANCO;
        } else {
            $fpas = array(
                '604', '647', '825', '833', '868',
            );

            // Sera zeros p/ FPAS 604, 647, 825, 833 e 868 (emp. domestico) e para a empresa optante pelo SIMPLES.
            if (((array_search($this->_fpas, $fpas) == true)
                ||($this->_simples == self::SIMPLES_OPTANTE))
            ) {
                $rat = parent::CARACTER_ZERO.parent::CARACTER_ZERO;
            }
        }

        // 1 casa para inteiro e 1 casa para decimal - Ex. 3.0% => 30.
        $this->atribuiDado($rat, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }


    /**
     * Atribui codigo centralizador.
     *
     * @param boolean $centraliza 		 Flag identificador para centralizar recolhimento.
     * @param string  $cnpjCentralizador CNPJ do centralizador.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     * 		   thalesvalias 	 - 31/01/2012 - Implementacao de validacao do codigo centralizador.
     *         alessandrobartels - 11/06/2012 - Criado codCentralizadorFGTS como atributo da classe.
     */
    private function _atribuiCodigoCentralizador(){

        $codRecol = array(
            '130', '135', '150', '155', '211', '317', '337', '608',
        );

        // Para os codigos de recolhimento 130, 135, 150, 155, 211, 317, 337, 608 o codigo centralizador sera 0.
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false) || ($this->_fpas === '868')) {
            $this->_codCentralizadorFGTS = parent::CARACTER_ZERO;
        } else {
            if ($this->_codCentralizadorFGTS === null) {
                $this->_codCentralizadorFGTS = parent::CARACTER_ZERO;
            }
        }

        $this->atribuiDado($this->_codCentralizadorFGTS, 1);
    }


    /**
     * Atribui opcao pelo SIMPLES.
     *
     * @param string $optanteSimples Opcao pelo SIMPLES.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     * 		   thalesvalias      - 31/01/2012 - Implementacao de validacao do SIMPLES.
     */
    private function _atribuiSimples($optanteSimples){
        $fpas = array(
            '582', '639', '663', '671', '680', '736', '868',
        );

        // Para os fpas 582, 639, 663, 671, 680, 736, 868 nao optante pelo simples.
        if ((array_search($this->_fpas, $fpas) !== false)) {
            $simples = self::SIMPLES_NAO_OPTANTE;
        } else {
            if ($this->_nrRecolhimento === '640') {
                $simples = parent::CARACTER_BRANCO;
            } else {
                if ($optanteSimples === parent::SIM) {
                    $simples = self::SIMPLES_OPTANTE;
                } else {
                    $simples = self::SIMPLES_NAO_OPTANTE;
                }
            }
        }

        $this->atribuiDado($simples, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }


    /**
     * Atribui codigo FPAS valido.
     *
     * @param string $fpas Codigo FPAS.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     */
    private function _atribuiFpas($fpas){
        $this->atribuiDado($fpas, 3, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui codigo de terceiros.
     *
     * @param string $codigoTerceiros Codigo de terceiros.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     */
    private function _atribuiCodigoTerceiros($codigoTerceiros){
        /* . N�o pode ser informado para os c�digos de recolhimento 145, 307, 317, 327, 337, 345, 640 e 660.
           � Deve ficar em branco quando o SIMPLES for igual a 2 , 3 ou 6.
           � Deve ser preenchido com zeros para o FPAS 582 quando a compet�ncia for maior ou igual a 10/1998.
           � Deve ser preenchido com zeros para FPAS 868 ( empregador dom�stico).
           � Deve estar contido na tabela de terceiros, inclusive zeros se SIMPLES for igual a 1, 4 ou 5.
           � Sempre que n�o informado o campo deve ficar em branco.
         */
        $codRecol = array(
            '145', '307', '317', '327', '337', '345', '640', '660',
        );

        $fpas = array(
            '582', '868',
        );

        if ((array_search($this->_nrRecolhimento, $codRecol) !== false)
            ||($this->_simples === self::SIMPLES_OPTANTE)
        ) {
            if (array_search($this->_fpas, $fpas) !== false) {
                $codigoTerceiros = parent::CARACTER_ZERO;
            } else {
                $codigoTerceiros = parent::CARACTER_BRANCO;
			}
        } else {
            if (array_search($this->_fpas, $fpas) !== false) {
                $codigoTerceiros = parent::CARACTER_ZERO;
            }
        }

        $this->atribuiDado($codigoTerceiros, 4, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }


    /**
     * Atribui codigo da GPS.
     *
     * @param string $codigoGPS Codigo da GPS.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     *         felipesantos - 07/05/2015
     */
    private function _atribuiCodigoGPS($codigoGPS){

        $codRecol = array(
            '115', '150', '211', '650'
        );

        if (array_search($this->_nrRecolhimento, $codRecol) == true) {
            if (is_null($this->getCodPagtoGPS())) {
                // caso seja o empregador rural, a gps deve ser gerada com o código 2208
                if ($this->_idTipoEmpr == 6) {
                    $codigoGPS = '2208';
                } else {
                    // caso seja o empregador domestico, a gps deve ser gerada com o código 1600
                    if ($this->_idTipoEmpr == 5) {
                        $codigoGPS = '1600';
                    } else {
                        // se for optante pelo simples, define o codigo 2003
                        if ($this->_simples == self::SIMPLES_OPTANTE) {
                            $codigoGPS = '2003';
                        }

                    }
                }
            } else {
                if ($this->_estrutura->historico->getCdfpas() == '868') {
                    $codGPS = array(
                        '1600', '1651'
                    );

                    if (array_search($this->getCodPagtoGPS(), $codGPS) == false ) {
                        tk_warning('Para FPAS 868 (empregador doméstico) só são aceitos os códigos 1600 e 1651 para pagamento da GPS.');
                    }
                }
                else {
                    $codigoGPS = $this->getCodPagtoGPS();
                }
            }
        }
        else {
            $codigoGPS = parent::CARACTER_BRANCO;
            //tk_warning(array_search($this->_nrRecolhimento, $codRecol));
        }

        //tk_warning('Recol.:'.$this->_nrRecolhimento.'/FPAS:'.$this->_estrutura->historico->getCdfpas().'/GPS:'.$codigoGPS);

        $this->atribuiDado($codigoGPS, 4, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }


    /**
     * Atribui percentual de filantropia.
     *
     * @param string $percentualFilantropia Percentual de Filantropia.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     */
    private function _atribuiPercentualFilantropia($percentualFilantropia){
        if ($this->_fpas !== '639') {
            $percentualFilantropia = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($percentualFilantropia, 5, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }


    /**
     * Atribui valor do Salario Familia.
     *
     * @param float $vrSalarioFamilia Valor do salario familia.
     *
     * @return void
     *
     * @author thalesvalias - 31/01/2012
     */
    private function _atribuiSalarioFamilia($vrSalarioFamilia) {

        $codRecol = array(
            '145', '307', '327', '345', '640', '650', '660',
        );

        // @todo Verificar leiaute - validar para a competencia 13.
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false) || ($this->_fpas === '868')) {
            $vrSalarioFamilia = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($vrSalarioFamilia, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui valor do Salario Maternidade.
     *
     * @param float $vrSalarioMaternidade Valor do salario Maternidade.
     *
     * @return void
     *
     * @author thalesvalias - 31/01/2012
     */
    private function _atribuiSalarioMaternidade($vrSalarioMaternidade) {

        $codRecol = array(
            '130', '135', '145', '211', '307', '317', '327', '337', '345', '640', '650', '660',
        );

        // @todo Verificar leiaute - validar para a competencia 13.
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false) || ($this->_fpas === '868')) {
            $vrSalarioMaternidade = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($vrSalarioMaternidade, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    public function alteraDadoRegistro10($registroMag, $dado) {
        //$registro, $dado, $posicaoInicial, $tamanhoCampo, $caracter, $orientacao

        $codRecol = array(
            '150', '155', '608', '907', '908', '910',
        );

        if ((array_search($this->_nrRecolhimento, $codRecol) === false)){
            // Salario Familia - Posicao Inicial = 235 Tamanho = 15. Subtrai 1 da posicao inicial, pois o array inicia em 0.
            $linha = $this->alteraDado(
                $registroMag->getDslinharegistro(), $dado[self::SEFIP_SALARIO_FAMILIA], (235 - 1), 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA
            );
            $registroMag->setDslinharegistro($linha);
        }

        // Salario Maternidade - Posicao Inicial = 250 Tamanho = 15. Subtrai 1 da posicao inicial, pois o array inicia em 0.
        $linha = $this->alteraDado(
            $registroMag->getDslinharegistro(), $dado[self::SEFIP_SALARIO_MATERNIDADE], (250 - 1), 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA
        );
        $registroMag->setDslinharegistro($linha);

        $registroMag->setVrsalfamilia($dado[self::SEFIP_SALARIO_FAMILIA]);
        $registroMag->setVrsalmaternidad($dado[self::SEFIP_SALARIO_MATERNIDADE]);

    }

    /**
     * Verifica CEP do endereço
     *
     * leonardocarvalho 21/08/2014
     */
    private function atribuiDadoCepEndereco()
    {
        $cep = $this->_endereco->getNrcependereco();

        if (!empty($cep) || $cep != '') {
            $this->atribuiDado($cep, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        } else {
            tk_warning("O CEP do endereço da estrutura responsável não foi informado.");
        }
    }

    /**
     * Verifica Código de Terceiros da estrutura responsável
     *
     * leonardocarvalho 21/08/2014
     */
    private function atribuiCodigoTerceiros()
    {
        $codigoTerceiros = $this->_estrutura->historico->getCdterceiro();

        if (!empty($codigoTerceiros) || $codigoTerceiros != '') {
            $this->_atribuiCodigoTerceiros($codigoTerceiros);
        } else {
            tk_warning("O Código de Terceiros da estrutura responsável não foi informado.");
        }

    }


}